@@ -1,7 +1,4 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
 	Copyright (C) 2006 Mic
     Copyright (C) 2009 CrazyMax 
 	Copyright (C) 2009 DeSmuME team
@@ -24,9 +21,8 @@
 */
 
 #include "../addons.h"
+#include <string>
 #include <string.h>
-#ifdef EXPERIMENTAL_GBASLOT
-
 #include "debug.h"
 #include <errno.h>
 #include <stdio.h>
@@ -59,6 +55,7 @@
 #include "../fs.h"
 #include "MMU.h"
 #include "NDSSystem.h"
+#include "../path.h"
 
 typedef struct {
 	int level,parent,filesInDir;
@@ -102,7 +99,7 @@ static const int lfnPos[13] = {1,3,5,7,9,14,16,18,20,22,24,28,30};
 #define BYTESPERCLUS (512*SECPERCLUS)
 #define DIRENTSPERCLUS ((BYTESPERCLUS)/32)
 
-static BOOT_RECORD MBR;
+BOOT_RECORD MBR;
 static DIR_ENT *files,*dirEntries,**dirEntryPtr;
 static FILE_INFO *fileLink,*dirEntryLink;
 static u32 filesysFAT,filesysRootDir,filesysData;
@@ -112,7 +109,6 @@ static DIR_ENT *extraDirEntries[SECPERFAT*256];
 static int numFiles,maxLevel,numRootFiles;
 static int *dirEntriesInCluster, clusterNum, firstDirEntCluster,
   lastDirEntCluster, lastFileDataCluster;
-static char *sRomPath;
 static int activeDirEnt=-1;
 static u32 bufferStart;
 static u32 fileStartLBA,fileEndLBA;
@@ -121,6 +117,8 @@ static FILE * hFile;
 static char fpath[255+1];
 static BOOL cflashDeviceEnabled = FALSE;
 
+static std::string sFlashPath;
+
 // ===========================
 BOOL	inited;
 
@@ -273,6 +271,7 @@ static void list_files(const char *filepath)
 	{
 		fname = (strlen(entry.cAlternateFileName)>0) ? entry.cAlternateFileName : entry.cFileName;
 		add_file(fname, &entry, fileLevel);
+		CFLASHLOG("cflash added %s\n",fname);
 
 		if (numFiles==MAXFILES-1) break;
 
@@ -281,7 +280,7 @@ static void list_files(const char *filepath)
 			if (strlen(fname)+strlen(filepath)+2 < 256) 
 			{
 				sprintf(SubDir, "%s%c%s", filepath, FS_SEPARATOR, fname);
-			list_files(SubDir);
+				list_files(SubDir);
 			}
 		}
 	}
@@ -298,7 +297,7 @@ static void list_files(const char *filepath)
 }
 
 // Set up the MBR, FAT and DIR_ENTs
-static BOOL cflash_build_fat( void)
+static BOOL cflash_build_fat()
 {
 	int i,j,k,l,
 	clust,numClusters,
@@ -309,12 +308,6 @@ static BOOL cflash_build_fat( void)
 	fileLevel = -1;
 	maxLevel  = -1;
 
-	if (CFlashUseRomPath)
-		sRomPath = pathToROM;
-	else
-		sRomPath = CFlashPath;
-		
-
 	files = (DIR_ENT *) malloc(MAXFILES*sizeof(DIR_ENT));
 	if (files == NULL) return FALSE;
 	fileLink = (FILE_INFO *) malloc(MAXFILES*sizeof(FILE_INFO));
@@ -336,8 +329,7 @@ static BOOL cflash_build_fat( void)
 		numExtraEntries[i] = 0;
 	}
 
-	//COMMENT OUT THIS LINE TO STOP THE IRRITATING FILESYSTEM SCANNING BEHAVIOR
-	list_files(sRomPath);
+	list_files(sFlashPath.c_str());
 
 	k            = 0;
 	clusterNum   = rootCluster = (SECRESV + SECPERFAT)/SECPERCLUS;
@@ -512,11 +504,21 @@ static BOOL cflash_init()
 	if (inited) return FALSE;
 	BOOL init_good = FALSE;
 
-	if (CFlashUsePath)
+	CFLASHLOG("CFlash_Mode: %d\n",CFlash_Mode);
+
+	if (CFlash_Mode == ADDON_CFLASH_MODE_RomPath)
+	{
+		sFlashPath = path.pathToRoms;
+		INFO("Using CFlash directory of rom: %s\n", sFlashPath.c_str());
+	}
+	else if(CFlash_Mode == ADDON_CFLASH_MODE_Path)
+	{
+		sFlashPath = CFlash_Path;
+		INFO("Using CFlash directory: %s\n", sFlashPath.c_str());
+	}
+
+	if(CFlash_IsUsingPath())
 	{
-		if (!strlen(CFlashPath)) CFlashUseRomPath = TRUE;
-		if (CFlashUseRomPath)
-			CFLASHLOG("Using CFlash directory of rom: %s\n", pathToROM);
 		cflashDeviceEnabled = FALSE;
 		currLBA = 0;
 
@@ -524,8 +526,10 @@ static BOOL cflash_init()
 			fclose(hFile);
 		activeDirEnt = -1;
 		fileStartLBA = fileEndLBA = 0xFFFFFFFF;
-		if (!cflash_build_fat())
+		if (!cflash_build_fat()) {
+			CFLASHLOG("FAILED cflash_build_fat\n");
 			return FALSE;
+		}
 		cf_reg_sts = 0x58;	// READY
 
 		cflashDeviceEnabled = TRUE;
@@ -533,9 +537,9 @@ static BOOL cflash_init()
 	}
 	else
 	{
-		if (!strlen(CFlashName)) return FALSE;
-		CFLASHLOG("Using CFlash disk image file %s\n", CFlashName);
-		disk_image = OPEN_FN( CFlashName, OPEN_MODE);
+		sFlashPath = CFlash_Path;
+		INFO("Using CFlash disk image file %s\n", sFlashPath.c_str());
+		disk_image = OPEN_FN( sFlashPath.c_str(), OPEN_MODE);
 
 		if ( disk_image != -1)
 		{
@@ -553,7 +557,7 @@ static BOOL cflash_init()
 		}
 		else
 			// TODO: create image if not exist
-			CFLASHLOG("Failed to open file %s: \"%s\"\n", CFlashName, strerror( errno));
+			CFLASHLOG("Failed to open file %s: \"%s\"\n", sFlashPath.c_str(), strerror( errno));
 	}
 
 	// READY
@@ -645,7 +649,7 @@ static u16 fread_buffered(int dirent,u32 cluster,u32 offset)
 	if (activeDirEnt != -1)
 		fclose(hFile);
 
-	strncpy(fpath,sRomPath,ARRAY_SIZE(fpath));
+	strncpy(fpath,sFlashPath.c_str(),ARRAY_SIZE(fpath));
 	strncat(fpath,DIR_SEP,ARRAY_SIZE(fpath)-strlen(fpath));
 
 	resolve_path(dirent);
@@ -653,6 +657,7 @@ static u16 fread_buffered(int dirent,u32 cluster,u32 offset)
 	fatstring_to_asciiz(dirent,fname,NULL);
 	strncat(fpath,fname,ARRAY_SIZE(fpath)-strlen(fpath));
 
+	CFLASHLOG("CFLASH Opening %s\n",fpath);
 	hFile = fopen(fpath, "rb");
 	if (!hFile) return 0;
 	bufferStart = offset;
@@ -686,7 +691,7 @@ static unsigned int cflash_read(unsigned int address)
 		case CF_REG_DATA:
 			if (cf_reg_cmd == CF_CMD_READ)
 			{
-				if (!CFlashUsePath)
+				if (!CFlash_IsUsingPath())
 				{
 					if ( disk_image != -1)
 					{
@@ -832,7 +837,7 @@ static void cflash_write(unsigned int address,unsigned int data)
 		case CF_REG_DATA:
 			if (cf_reg_cmd == CF_CMD_WRITE)
 			{
-				if (!CFlashUsePath) 
+				if (!CFlash_IsUsingPath()) 
 				{
 					sector_data[sector_write_index] = (data >> 0) & 0xff;
 					sector_data[sector_write_index + 1] = (data >> 8) & 0xff;
@@ -862,9 +867,10 @@ static void cflash_write(unsigned int address,unsigned int data)
 						}
 
 						CFLASHLOG("Wrote %u bytes\n", written);
+					
+						currLBA += 512;
+						sector_write_index = 0;
 					}
-					currLBA += 512;
-					sector_write_index = 0;
 				}
 				else		// TODO: write to real partition
 				{
@@ -908,11 +914,11 @@ static void cflash_write(unsigned int address,unsigned int data)
 static void cflash_close( void) 
 {
 	if (!inited) return;
-	if (!CFlashUsePath)
+	if (!CFlash_IsUsingPath())
 	{
-		if ( disk_image != -1)
+		if (disk_image != -1)
 		{
-			CLOSE_FN( disk_image);
+			CLOSE_FN(disk_image);
 			disk_image = -1;
 		}
 	}
@@ -947,8 +953,6 @@ static void cflash_close( void)
 
 static BOOL CFlash_init(void)
 {
-	CFlashUseRomPath = TRUE;
-	CFlashUsePath = TRUE;
 	return TRUE;
 }
 
@@ -1001,23 +1005,6 @@ static void CFlash_info(char *info)
 {
 	strcpy(info, "Compact Flash memory in slot");
 }
-#else
-static BOOL CFlash_init(void) { return TRUE; }
-static void CFlash_reset(void) {}
-static void CFlash_close(void) {}
-static void CFlash_config(void){}
-static void CFlash_write08(u32 adr, u8 val){}
-static void CFlash_write16(u32 adr, u16 val){}
-static void CFlash_write32(u32 adr, u32 val){}
-static u8   CFlash_read08(u32 adr){return (0);}
-static u16  CFlash_read16(u32 adr){return (0);}
-static u32  CFlash_read32(u32 adr){return (0);}
-
-static void CFlash_info(char *info)
-{
-	strcpy(info, "Compact Flash memory in slot");
-}
-#endif
 
 ADDONINTERFACE addonCFlash = {
 				"Compact Flash",
