@@ -293,12 +293,12 @@ void set_mouse_coord(signed long x,signed long y)
 void update_keypad(u16 keys)
 {
 #ifdef WORDS_BIGENDIAN
-  ARM9Mem.ARM9_REG[0x130] = ~keys & 0xFF;
-  ARM9Mem.ARM9_REG[0x131] = (~keys >> 8) & 0x3;
+  MMU.ARM9_REG[0x130] = ~keys & 0xFF;
+  MMU.ARM9_REG[0x131] = (~keys >> 8) & 0x3;
   MMU.ARM7_REG[0x130] = ~keys & 0xFF;
   MMU.ARM7_REG[0x131] = (~keys >> 8) & 0x3;
 #else
-  ((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] = ~keys & 0x3FF;
+  ((u16 *)MMU.ARM9_REG)[0x130>>1] = ~keys & 0x3FF;
   ((u16 *)MMU.ARM7_REG)[0x130>>1] = ~keys & 0x3FF;
 #endif
   /* Update X and Y buttons */
@@ -312,9 +312,9 @@ u16 get_keypad( void)
   keypad = ~MMU.ARM7_REG[0x136];
   keypad = (keypad & 0x3) << 10;
 #ifdef WORDS_BIGENDIAN
-  keypad |= ~(ARM9Mem.ARM9_REG[0x130] | (ARM9Mem.ARM9_REG[0x131] << 8)) & 0x3FF;
+  keypad |= ~(MMU.ARM9_REG[0x130] | (MMU.ARM9_REG[0x131] << 8)) & 0x3FF;
 #else
-  keypad |= ~((u16 *)ARM9Mem.ARM9_REG)[0x130>>1] & 0x3FF;
+  keypad |= ~((u16 *)MMU.ARM9_REG)[0x130>>1] & 0x3FF;
 #endif
   return keypad;
 }
