@@ -22,10 +22,6 @@
 
 #include "types.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #define FS_IS_DIR 1
 
 #define FS_ERR_UNKNOWN -1
@@ -45,8 +41,4 @@ int FsReadNext(void * search, FsEntry * entry);
 void FsClose(void * search);
 int FsError(void);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
