@@ -19,13 +19,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 #include "callbacks.h"
 #include "callbacks_IO.h"
 #include "dTools/callbacks_dtools.h"
 #include "globals.h"
 #include "keyval_names.h"
 #include "rasterize.h"
+#include "desmume.h"
 
 #ifdef GDB_STUB
 #include "../gdbstub.h"
@@ -37,10 +38,6 @@
 #include "gdk_3Demu.h"
 #endif
 
-/*
- * The frame limiter semaphore
- */
-//SDL_sem *glade_fps_limiter_semaphore;
 int glade_fps_limiter_disabled = 0;
 
 GtkWidget * pWindow;
@@ -67,7 +64,7 @@ const u16 gtk_kb_cfg[NB_KEYS] =
 
 SoundInterface_struct *SNDCoreList[] = {
 &SNDDummy,
-&SNDFile,
+&SNDDummy,
 &SNDSDL,
 NULL
 };
@@ -92,6 +89,7 @@ struct configured_features {
   int opengl_2d;
   int engine_3d;
   int disable_limiter;
+  int savetype;
 
   u16 arm9_gdb_port;
   u16 arm7_gdb_port;
@@ -113,6 +111,7 @@ init_configured_features( struct configured_features *config) {
   config->engine_3d = 1;
 
   config->disable_limiter = 0;
+  config->savetype = 0;
 
   config->nds_file = NULL;
 
@@ -148,8 +147,18 @@ fill_configured_features( struct configured_features *config,
                          2 = gtkglext off-screen 3d opengl\n\n"));
 #endif
       g_print( _("\
-   --disable-limiter   Disables the 60 fps limiter\n\
-   \n\
+   --disable-limiter   Disables the 60 fps limiter\n\n"));
+      g_print( _("\
+   --save-type=TYPE    Selects savetype:\n\
+                         0 = Autodetect (default)\n\
+                         1 = EEPROM 4kbit\n\
+                         2 = EEPROM 64kbit\n\
+                         3 = EEPROM 512kbit\n\
+                         4 = FRAM 256kbit\n\
+                         5 = FLASH 2mbit\n\
+                         6 = FLASH 4mbit\n\
+   \n"));
+      g_print( _("\
    --fwlang=LANG       Set the language in the firmware, LANG as follows:\n\
                          0 = Japanese\n\
                          1 = English\n\
@@ -203,6 +212,18 @@ fill_configured_features( struct configured_features *config,
         good_args = 0;
       }
     }
+    else if ( strncmp( argv[i], "--save-type=", 12) == 0) {
+      char *end_char;
+      int type = strtoul( &argv[i][12], &end_char, 10);
+
+      if ( type >= 0 && type <= 6) {
+        config->savetype = type;
+      }
+      else {
+        g_printerr( _("select savetype from 0 to 6; use --help option for details\n"));
+        good_args = 0;
+      }
+    }
     else if ( strncmp( argv[i], "--fwlang=", 9) == 0) {
       char *end_char;
       int lang = strtoul( &argv[i][9], &end_char, 10);
@@ -401,28 +422,6 @@ joinThread_gdb( void *thread_handle) {
 #endif
 
 
-
-/** 
- * A SDL timer callback function. Signals the supplied SDL semaphore
- * if its value is small.
- * 
- * @param interval The interval since the last call (in ms)
- * @param param The pointer to the semaphore.
- * 
- * @return The interval to the next call (required by SDL)
- */
-/*static Uint32
-glade_fps_limiter_fn( Uint32 interval, void *param) {
-  SDL_sem *sdl_semaphore = (SDL_sem *)param;
-
-  /* signal the semaphore if it is getting low *-/
-  if ( SDL_SemValue( sdl_semaphore) < 4) {
-    SDL_SemPost( sdl_semaphore);
-  }
-
-  return interval;
-}*/
-
 /* ***** ***** MAIN ***** ***** */
 
 static int
@@ -447,6 +446,7 @@ common_gtk_glade_main( struct configured_features *my_config) {
         if ( my_config->firmware_language != -1) {
           fw_config.language = my_config->firmware_language;
         }
+        desmume_savetype(my_config->savetype);
 
 #ifdef GTKGLEXT_AVAILABLE
 // check if you have GTHREAD when running configure script
@@ -525,6 +525,13 @@ common_gtk_glade_main( struct configured_features *my_config) {
 	pDrawingArea  = glade_xml_get_widget(xml, "wDraw_Main");
 	pDrawingArea2 = glade_xml_get_widget(xml, "wDraw_Sub");
 
+    {
+      char wdgName[40];
+      snprintf(wdgName, 39, "savetype%d", my_config->savetype+1);
+      GtkWidget * wdgt = glade_xml_get_widget(xml, wdgName);
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (wdgt), TRUE);
+    }
+
 	/* connect the signals in the interface */
 	glade_xml_signal_autoconnect_StringObject(xml);
 	glade_xml_signal_autoconnect_StringObject(xml_tools);
@@ -573,33 +580,10 @@ common_gtk_glade_main( struct configured_features *my_config) {
         /* setup the frame limiter and indicate if it is disabled */
         glade_fps_limiter_disabled = my_config->disable_limiter;
 
-    /*    if ( !glade_fps_limiter_disabled) {
-          /* create the semaphore used for fps limiting *-/
-          glade_fps_limiter_semaphore = SDL_CreateSemaphore( 1);
-
-          /* start a SDL timer for every FPS_LIMITER_FRAME_PERIOD
-           * frames to keep us at 60 fps *-/
-          limiter_timer = SDL_AddTimer( 16 * FPS_LIMITER_FRAME_PERIOD,
-                                        glade_fps_limiter_fn,
-                                        glade_fps_limiter_semaphore);
-          if ( limiter_timer == NULL) {
-            fprintf( stderr, _("Error trying to start FPS limiter timer: %s\n"),
-                     SDL_GetError());
-            SDL_DestroySemaphore( glade_fps_limiter_semaphore);
-            glade_fps_limiter_disabled = 1;
-          }
-        }*/
-
 	/* start event loop */
 	gtk_main();
 	desmume_free();
 
-      /*  if ( !glade_fps_limiter_disabled) {
-          /* tidy up the FPS limiter timer and semaphore *-/
-          SDL_RemoveTimer( limiter_timer);
-          SDL_DestroySemaphore( glade_fps_limiter_semaphore);
-        }*/
-
         /* Unload joystick */
         uninit_joy();
 
