@@ -24,18 +24,9 @@
 #include <math.h>
 #include <assert.h>
 #include "matrix.h"
+#include "MMU.h"
 
-extern "C" {
-
-
-void MatrixInit  (float *matrix)
-{
-	memset (matrix, 0, sizeof(float)*16);
-	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1.f;
-}
-
-#ifdef NOSSE2
-void MATRIXFASTCALL MatrixMultVec4x4 (const float *matrix, float *vecPtr)
+void _NOSSE_MatrixMultVec4x4 (const float *matrix, float *vecPtr)
 {
 	float x = vecPtr[0];
 	float y = vecPtr[1];
@@ -48,7 +39,17 @@ void MATRIXFASTCALL MatrixMultVec4x4 (const float *matrix, float *vecPtr)
 	vecPtr[3] = x * matrix[3] + y * matrix[7] + z * matrix[11] + w * matrix[15];
 }
 
-void MATRIXFASTCALL MatrixMultVec3x3 (const float *matrix, float *vecPtr)
+
+//-------------------------
+//switched SSE functions: implementations for no SSE
+#ifndef ENABLE_SSE
+void MatrixMultVec4x4 (const float *matrix, float *vecPtr)
+{
+	_NOSSE_MatrixMultVec4x4(matrix, vecPtr);
+}
+
+
+void MatrixMultVec3x3 (const float *matrix, float *vecPtr)
 {
 	float x = vecPtr[0];
 	float y = vecPtr[1];
@@ -59,7 +60,7 @@ void MATRIXFASTCALL MatrixMultVec3x3 (const float *matrix, float *vecPtr)
 	vecPtr[2] = x * matrix[2] + y * matrix[6] + z * matrix[10];
 }
 
-void MATRIXFASTCALL MatrixMultiply (float *matrix, const float *rightMatrix)
+void MatrixMultiply (float *matrix, const float *rightMatrix)
 {
 	float tmpMatrix[16];
 
@@ -86,7 +87,7 @@ void MATRIXFASTCALL MatrixMultiply (float *matrix, const float *rightMatrix)
 	memcpy (matrix, tmpMatrix, sizeof(float)*16);
 }
 
-void MATRIXFASTCALL MatrixTranslate	(float *matrix, const float *ptr)
+void MatrixTranslate	(float *matrix, const float *ptr)
 {
 	matrix[12] += (matrix[0]*ptr[0])+(matrix[4]*ptr[1])+(matrix[ 8]*ptr[2]);
 	matrix[13] += (matrix[1]*ptr[0])+(matrix[5]*ptr[1])+(matrix[ 9]*ptr[2]);
@@ -94,7 +95,7 @@ void MATRIXFASTCALL MatrixTranslate	(float *matrix, const float *ptr)
 	matrix[15] += (matrix[3]*ptr[0])+(matrix[7]*ptr[1])+(matrix[11]*ptr[2]);
 }
 
-void MATRIXFASTCALL MatrixScale (float *matrix, const float *ptr)
+void MatrixScale (float *matrix, const float *ptr)
 {
 	matrix[0]  *= ptr[0];
 	matrix[1]  *= ptr[0];
@@ -111,9 +112,16 @@ void MATRIXFASTCALL MatrixScale (float *matrix, const float *ptr)
 	matrix[10] *= ptr[2];
 	matrix[11] *= ptr[2];
 }
+
 #endif //switched c/asm functions
 //-----------------------------------------
 
+void MatrixInit  (float *matrix)
+{
+	memset (matrix, 0, sizeof(float)*16);
+	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1.f;
+}
+
 void MatrixTranspose(float *matrix)
 {
 	float temp;
@@ -127,20 +135,15 @@ void MatrixTranspose(float *matrix)
 #undef swap
 }
 
-void MATRIXFASTCALL MatrixIdentity	(float *matrix) //============== TODO
+void MatrixIdentity	(float *matrix)
 {
-	//memset (matrix, 0, sizeof(float)*16);
-	//this is fastest for SSE2 i think.
-	//study code generation and split into sse2 specific module later
-	for(int i=0;i<16;i++)
-		matrix[i] = 0.0f;
-	//matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0f;
-	//matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0f;
-	//matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0f;
+	matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0f;
+	matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0f;
+	matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0f;
 	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1.f;
 }
 
-float MATRIXFASTCALL MatrixGetMultipliedIndex (int index, float *matrix, float *rightMatrix)
+float MatrixGetMultipliedIndex (int index, float *matrix, float *rightMatrix)
 {
 	int iMod = index%4, iDiv = (index>>2)<<2;
 
@@ -148,26 +151,51 @@ float MATRIXFASTCALL MatrixGetMultipliedIndex (int index, float *matrix, float *
 			(matrix[iMod+8]*rightMatrix[iDiv+2])+(matrix[iMod+12]*rightMatrix[iDiv+3]);
 }
 
-void MATRIXFASTCALL MatrixSet (float *matrix, int x, int y, float value)	// TODO
+void MatrixSet (float *matrix, int x, int y, float value)	// TODO
 {
 	matrix [x+(y<<2)] = value;
 }
 
-void MATRIXFASTCALL MatrixCopy (float* matrixDST, const float* matrixSRC)
+void MatrixCopy (float* matrixDST, const float* matrixSRC)
 {
-	memcpy ((void*)matrixDST, matrixSRC, sizeof(float)*16);
+	matrixDST[0] = matrixSRC[0];
+	matrixDST[1] = matrixSRC[1];
+	matrixDST[2] = matrixSRC[2];
+	matrixDST[3] = matrixSRC[3];
+	matrixDST[4] = matrixSRC[4];
+	matrixDST[5] = matrixSRC[5];
+	matrixDST[6] = matrixSRC[6];
+	matrixDST[7] = matrixSRC[7];
+	matrixDST[8] = matrixSRC[8];
+	matrixDST[9] = matrixSRC[9];
+	matrixDST[10] = matrixSRC[10];
+	matrixDST[11] = matrixSRC[11];
+	matrixDST[12] = matrixSRC[12];
+	matrixDST[13] = matrixSRC[13];
+	matrixDST[14] = matrixSRC[14];
+	matrixDST[15] = matrixSRC[15];
+
 }
 
-int MATRIXFASTCALL MatrixCompare (const float* matrixDST, const float* matrixSRC)
+int MatrixCompare (const float* matrixDST, const float* matrixSRC)
 {
 	return memcmp((void*)matrixDST, matrixSRC, sizeof(float)*16);
 }
 
+void MatrixStackInit(MatrixStack *stack)
+{
+	for (int i = 0; i < stack->size; i++)
+	{
+		MatrixInit(&stack->matrix[i*16]);
+	}
+	stack->position = 0;
+}
+
 void MatrixStackSetMaxSize (MatrixStack *stack, int size)
 {
 	int i;
 
-	stack->size = size;
+	stack->size = (size + 1);
 
 	if (stack->matrix != NULL) {
 		free (stack->matrix);
@@ -190,12 +218,17 @@ MatrixStack::MatrixStack(int size)
 
 void MatrixStackSetStackPosition (MatrixStack *stack, int pos)
 {
+	//printf("SetPosition: %d by %d",stack->position,pos);
 	stack->position += pos;
 
-	if (stack->position < 0)
-		stack->position = 0;
-	else if (stack->position > stack->size)	
-		stack->position = stack->size+1;
+	//this wraparound behavior fixed sims apartment pets which was constantly going up to 32
+	s32 newpos = stack->position;
+	stack->position &= (stack->size);
+
+	if(newpos != stack->position)
+		MMU_new.gxstat.se = 1;
+
+	//printf(" to %d (size %d)\n",stack->position,stack->size);
 }
 
 void MatrixStackPushMatrix (MatrixStack *stack, const float *ptr)
@@ -320,5 +353,4 @@ void Vector4Copy(float *dst, const float *src)
 	dst[3] = src[3];
 }
 
-} //extern "C"
 
