@@ -1,8 +1,4 @@
-/*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
-	Copyright 2009 DeSmuME team
+/*  Copyright 2009 DeSmuME team
 
     This file is part of DeSmuME
 
